# ------------------------------------------------------------------------
#
#	MakeSrcdepgen.pl
#
# Notes:
#   - Find's #include statements and generate dependency list
#
#   Date			  Name		   Vers		Change
#	----------	 ------		   ----	-------------------------------
#	2009-02-22   chenyanlv	 1.0   Init Version
# ------------------------------------------------------------------------

use strict;
#use warnings;
use File::Basename;

# ------------------------------------------------------------------------
# BuildDir
#   User specified directory for targets
# ------------------------------------------------------------------------
my $BuildDir = "\$(TARGETDIR)";

# ------------------------------------------------------------------------
my %IncFiles  = ();
my %SrcFiles  = ();
my %TargDeps  = ();
my %IncDeps   = ();

# ------------------------------------------------------------------------
# Patterns
# ------------------------------------------------------------------------

my $IncPattern    = '^(#|\s*)*include\s+(\"|<)(.*)(\"|>)';

#-------------------------------------------------------------------------------

my($BuildID,$OUTDIR)=@ARGV;

my (@Qincfiles,$Qincfiles);
my (@Qsrcfiles,$Qsrcfiles);
my (@Metaincfiles,$Metaincfiles);
my (@Metasrcfiles,$Metasrcfiles);
my (@AllIncs,@AllSrcs);

my (@BrowserIncs,$BrowserIncs);
my %BrowserIncs = ();

my %MacroHeader = ( ADC_CHANNELS_H => '"adc_channels_surf6250.h"',
                    BOOTHW_MSM_H   => '"boothw_6280.h"',
                    GPIO_INT_MSM_H => '"gpio_int_6280.h"',
                    TRAMP_MSM_H    => '"tramp_msm.h"',
                    TRAMPI_MSM_H   => '"trampi_msm.h"',
                    CLKRGM_H       => '"clkrgm_msm.h"',
                    CLKRGMI_H      => '"clkrgmi_msm.h"',
                    DOWNLOAD_H     => '"dload_6260.h"',
                    CPLD_MSM_H     => '"cpld_6280.h"',
                    ADIE_I_H       => '"adie_sbi.h"',
                    BSP_RF_H       => '"bsp_rf_zrf6300.h"',
                    HS_H           => '"hs_mb6280.h"',
                    SBI_MSM_H      => '"sbi_6280.h"',
                    CHG_TARGET_H   => '"chg_pm6650_config.h"'
                    );
# ------------------------------------------------------------------------
# SUBROUTINE: ReadFiletoHash
#
# ------------------------------------------------------------------------
sub ReadFiletoHash
{
	my( $name, $path);
	my $lcname;
	#read include files
	open(INCFH,"$OUTDIR\/MetaFile\/Qincfiles.txt")||die "$!\n";
	while(<INCFH>)
	{
		chomp;
		$Qincfiles = $Qincfiles.$_;
	}
	close(INCFH);
	@Qincfiles = split(/ /,$Qincfiles);
	
	open(INCFH,"$OUTDIR\/MetaFile\/MetaFilesInc.txt")||die "$!\n";
	while(<INCFH>)
	{
		chomp;
		$Metaincfiles = $Metaincfiles.$_." ";
	}
	close(INCFH);
	@Metaincfiles = split(/ /,$Metaincfiles);
	
	@AllIncs = (@Qincfiles,@Metaincfiles);
	
	foreach (@AllIncs)
	{
		chomp;
		$name= basename($_);
		$lcname = lc($name);
		$path= dirname($_);
		if((exists $IncFiles{$lcname}) and (lc($IncFiles{$lcname}) ne lc($_)))
		{
			my $prev_path = dirname($IncFiles{$lcname});
			print "#Replace $name\n";
			print "#   --From: $prev_path\n";
			print "#   --To:   $path\n";
		}
		$IncFiles{$lcname}=$_;	
	}
	
	#read browser include files
#	open(INCFH,"Browserincfiles.txt")||die "$!\n";
#	while(<INCFH>)
#	{
#		chomp;
#		$BrowserIncs = $BrowserIncs.$_;
#	}
#	close(INCFH);
#	@BrowserIncs = split(/ /,$BrowserIncs);
#	
#	foreach (@BrowserIncs)
#	{
#		chomp;
#		$name= basename($_);
#		$lcname = lc($name);
#		
#		push @{$BrowserIncs{$lcname}},$_;
#	}

	#read source files
	open(SRCFH,"$OUTDIR\/MetaFile\/Qsrcfiles.txt")||die "$!\n";
	while(<SRCFH>)
	{
		chomp;
		$Qsrcfiles = $Qsrcfiles.$_;
	}
	close(SRCFH);
	@Qsrcfiles = split(/ /,$Qsrcfiles);
	
	open(SRCFH,"$OUTDIR\/MetaFile\/MetaFilesSrc.txt")||die "$!\n";
	while(<SRCFH>)
	{
		chomp;
		$Metasrcfiles = $Metasrcfiles.$_." ";
	}
	close(SRCFH);
	@Metasrcfiles = split(/ /,$Metasrcfiles);
	
	@AllSrcs = (@Qsrcfiles,@Metasrcfiles);
	
	foreach (@AllSrcs)
	{
		chomp;
		$name= basename($_);
		$path= dirname($_);
		if((exists $SrcFiles{$name}) and (lc($SrcFiles{$name}) ne lc($_)))
		{
			my $prev_path = dirname($SrcFiles{$name});
			print "#Replace $name\n";
			print "#   --From: $prev_path\n";
			print "#   --To:   $path\n";
		}
		$SrcFiles{$name}=$_;	
	}
	close(SRCFH);
}
# ------------------------------------------------------------------------
# SUBROUTINE: ParseForIncludes
#
# ------------------------------------------------------------------------
sub ParseForIncludes
{
   my( $filename) = @_;
	
   #local $CURFILE;

   if(!open(CURFILE, "$IncFiles{$filename}"))
   {
   		if(!open( CURFILE, "$SrcFiles{$filename}")) 
   		{
   			foreach my $file (@{$BrowserIncs{$filename}})
   			{
   				open(CURFILE,"$file")|| die "Couldn't open $file\n";
   				while( <CURFILE> )
   				{
      			chomp;

      			if( /^(#|\s*)*include/ )
      			{
         			if(/_H/)
         			{
         				foreach my $Macro (keys %MacroHeader)
         				{
         					if(/$Macro/i)
         					{
         						s/$Macro/$MacroHeader{$Macro}/g;
         						last;
         					}
         				}
        			}
         			/$IncPattern/;
         			push @{$IncDeps{$filename}}, lc($3);
      			}
   				}	
   				close( CURFILE );
   			}
   			return;
   		}
   	}
   while( <CURFILE> )
   {
      chomp;

      if( /^(#|\s*)*include/ )
      {
         if(/_H/)
         {
         		foreach my $Macro (keys %MacroHeader)
         		{
         			if(/$Macro/i)
         			{
         				s/$Macro/$MacroHeader{$Macro}/g;
         				last;
         			}
         		}
        	}
         /$IncPattern/;
         push @{$IncDeps{$filename}}, lc($3);
      }
   }
   close( CURFILE ); 
}

# ------------------------------------------------------------------------
# SUBROUTINE: ParseDeps
#
# ------------------------------------------------------------------------
sub ParseDeps
{
   my $name;
	 foreach $name ( keys %SrcFiles )
   { 
      ParseForIncludes($name);
   }
   foreach $name ( keys %IncFiles )
   { 
      ParseForIncludes($name);
   
      # CUSTOMER.H
      #    $(TARGETDIR)/%.o : customer.h
      #       should also get
      #    $(TARGETDIR)/%.o : $(CUSTFILE)
      #    $(TARGETDIR)/%.o : includes from $(CUSTFILE)
      #
      #    Add the cust<build_id>.h file 
      #
      if( $name eq "customer.h" && $BuildID ne "" )
      {
         my $custbid_name = "cust" . lc( $BuildID ) . ".h";
 
         push @{$IncDeps{$name}}, $custbid_name;
      }
    }
    
    #Browser include files
    foreach $name ( keys %BrowserIncs )
   	{ 
      ParseForIncludes($name);
      if( $name eq "customer.h" && $BuildID ne "" )
      {
         my $custbid_name = "cust" . lc( $BuildID ) . ".h";
 
         push @{$IncDeps{$name}}, $custbid_name;
      }
   	}
}

# ------------------------------------------------------------------------
# SUBROUTINE: ReconcileFile 
#
# ------------------------------------------------------------------------
sub ReconcileFile
{
   my ($file) = @_;

   #remove "//" and "/*"
   if(($file =~ m/\/\//) or ($file =~ m/\/\*/))
   {
   		substr($file,index($file,"//")) = "";
   		substr($file,index($file,"/\*")) = "";
   		substr($file,index($file,"\"")) = "";
   		#substr($file,index($file,"\>")) = "";
   }
   
   #remove space
   $file =~ s/ //g;
   # "xxx/xxx.x" -> "xxx.x"
   if($file =~ m/\//)
   {
   		$file = substr($file,rindex($file,"/")+1);
   }
   elsif($file =~ m/\\/)
   {
   		$file = substr($file,rindex($file,"\\")+1);
   }
   
   if(( !exists $IncFiles{lc($file)} ) and ( !exists $BrowserIncs{lc($file)} ))
   {
			 return "";
   }
   return $file;
}

# ------------------------------------------------------------------------
# SUBROUTINE: CalcDeps
#
# ------------------------------------------------------------------------
sub CalcDeps
{
   my ( $lhv, $targ ) = @_;
   my $dep;

   foreach $dep ( @{$IncDeps{$lhv}} )
   {
      if($TargDeps{$targ}{$dep} == 1)
      {
      	next;
      }
      
      # Assemble RHV (dependency)
      #
      my $rhv = ReconcileFile( $dep );
      
      if(( $rhv ne "" ) and ( $TargDeps{$targ}{$rhv} == 1 ))
      {
         next;
      }
      
      elsif( $rhv ne "" )
      {
         if(exists $IncFiles{lc($rhv)})
         {
         		print DEPFILE "$targ: $IncFiles{lc($rhv)}\n";
         }
         else
         {
         	foreach (@{$BrowserIncs{lc($rhv)}})
				 	{
         		print DEPFILE "$targ: $_\n";
         	}
         }
         
         $TargDeps{$targ}{$rhv} = 1; 
         
         CalcDeps( $rhv, $targ );   
      }
      else
      {
         if($dep ne "")
         {
         		print DEPFILE "# DEP (File not in build tree): $dep\n";
         }
         $TargDeps{$targ}{$dep} = 1;
      }
   }
}

# ------------------------------------------------------------------------
# SUBROUTINE: CalcAll 
#
# ------------------------------------------------------------------------
sub CalcAll
{
   my $file;
   foreach $file ( keys %SrcFiles )
   {
      undef %TargDeps;
      # Assemble LHV (target)
      #
      # (.c, cpp, .s, etc.) $builddir$file
      #
      # where $file for .c, .s, and .cpp is converted to .o
      #

      my $lhv;
      my $lsrcdep;
   		if($file=~ m/(\.c|\.cpp|\.s)$/i)
   		{
         $lhv = "$BuildDir/$file";
         $lhv =~ s/(\.cpp)|(\.c)|(\.s)/\.o/g;
         $lsrcdep = "$OUTDIR\/dep\/$file";
         $lsrcdep =~ s/(\.cpp)|(\.c)|(\.s)/\.dep/g;
      }
      
      open (DEPFILE, ">$lsrcdep") || die ("Could not open $lsrcdep");
      CalcDeps( $file, $lhv );
      close(DEPFILE);
   }
}

# ------------------------------------------------------------------------
# MAIN SCRIPT
#
# ------------------------------------------------------------------------

ReadFiletoHash();

#if some include files are modified,generate dependency list
ParseDeps();

CalcAll();

1;
